(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - MODUL  DEP                                       13.03.93  *)
(*                                                                          *)
(*  (c) Markus Neher    Institute for Applied Mathematics                   *)
(*                      D-76128 Karlsruhe University, Germany               *)
(*                      e-mail: ae16@rz.uni-karlsruhe.de                    *)
(*                                                                          *)
(*  BERECHNUNG VON NHERUNGEN UND EINSCHLIESSUNGEN VON EIGENWERTEN UND      *)
(*  EIGENFUNKTIONEN DER STURM-LIOUVILLE-GLEICHUNG ( DIREKTES PROBLEM )      *)
(*                                                                          *)
(*                  - U" + P(X)  U(X) =   U(X)                           *)
(*                         U(0) = U() = 0                                  *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

MODULE dep;

USE i_ari , mv_ari , mvi_ari;
USE iep_rawa , iep_iawa , iep_tayk , seig , seig_app , splines , service;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EWT_RITZ                                13.03.93  *)
(*                                                                          *)
(*  RITZ-VERFAHREN ZUR BESTIMMUNG DER SLP-EIGENWERTE EINER                  *)
(*  ENDLICHEN COSINUSREIHE MIT GERADZAHLIGEN FREQUENZEN                     *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

GLOBAL FUNCTION EWT_RITZ( ANZ : INTEGER; KOEFVEC : RVECTOR ) :
       RVECTOR[0..2*ANZ-1];

(*  NEBEN DEN EIGENWERTEN WERDEN IN HEIGVEC AUCH DIE EIGENVEKTOREN  *)
(*  BERECHNET, DIESE WERDEN ABER DERZEIT NICHT WEITER VERWENDET     *)

(*  IN KOEFVEC[I] STEHT DIE HALBE AMPLITUDE DES TERMS  cos( 2i x )  *)

(*  ANZ  =  ANZAHL DER ANSATZFUNKTIONEN  SIN( KX )  BEIM RITZVERF.  *)
(*          JE GERADE UND UNGERADE !                                *)

(*--------------------------------------------------------------------------*)

VAR I , J , K         : INTEGER;
    H1                : REAL;
    VALID             : BOOLEAN;
    HEIGVAL , VR1     : RVECTOR[1..ANZ];
    EIGVAL            : RVECTOR[0..2*ANZ-1];
    RITZMAT , HEIGVEC : RMATRIX[1..ANZ,1..ANZ];

(*--------------------------------------------------------------------------*)

BEGIN

   (*  ERSTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  *)
   (*  MIT GERADEM INDEX :  RITZ-MATRIX BESETZEN        *)

   (*  IM MATRIXARGUMENT GILT  "I = 2 * I - 1, J = 2 * J - 1"  *)
   (*  IM VEKTORARGUMENT GILT ABER  "I = 2 * I , J = 2 * J" !  *)

   FOR I := 1 TO ANZ DO BEGIN
       RITZMAT[I,I] := SQR( 2 * I - 1 ) + KOEFVEC[0] - KOEFVEC[2*I-1];
       FOR J := 1 TO I - 1 DO BEGIN
           RITZMAT[I,J] := KOEFVEC[I-J] - KOEFVEC[I+J-1];
           RITZMAT[J,I] := RITZMAT[I,J];
       END;
   END;

   (*  ZWEITER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  MIT             *)
   (*  GERADEM INDEX :  MODUL SEIG_APP ZUR EIGENWERTBERECHNUNG AUFRUFEN  *)

   SEIG_APP( RITZMAT , HEIGVAL , HEIGVEC );

   (*  EIGENWERTE SORTIEREN  *)

   K := 0;
   REPEAT VALID := TRUE;
          K := K + 1;
          FOR I := ANZ DOWNTO K + 1 DO
              IF HEIGVAL[I] < HEIGVAL[I-1] THEN BEGIN
                 VALID := FALSE;
                 H1 := HEIGVAL[I];
              (* VR1 := HEIGVEC[*,I]; *)
                 HEIGVAL[I] := HEIGVAL[I-1];
              (* HEIGVEC[*,I] := HEIGVEC[*,I-1]; *)
                 HEIGVAL[I-1] := H1;
              (* HEIGVEC[*,I-1] := VR1; *)
              END;
   UNTIL VALID;

   FOR I := 0 TO ANZ - 1 DO BEGIN
       EIGVAL[2*I] := HEIGVAL[I+1];
   (*  FOR J := 0 TO N DO                      *)
   (*      EIGVEC[J,2*I] := HEIGVEC[J+1,I+1];  *)
   END;

   (*  ERSTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)   *)
   (*  MIT UNGERADEM INDEX :  RITZ-MATRIX BESETZEN       *)

   (*  IM MATRIXARGUMENT GILT  "I = 2 * I , J = 2 * J"  *)
   (*  IM VEKTORARGUMENT GILT  "I = 2 * I , J = 2 * J"  *)

   FOR I := 1 TO ANZ DO BEGIN
       RITZMAT[I,I] := SQR( 2 * I ) + KOEFVEC[0] - KOEFVEC[2*I];
       FOR J := 1 TO I - 1 DO BEGIN
           RITZMAT[I,J] := KOEFVEC[I-J] - KOEFVEC[I+J];
           RITZMAT[J,I] := RITZMAT[I,J];
       END;
   END;

   (*  ZWEITER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  MIT               *)
   (*  UNGERADEM INDEX :  MODUL SEIG_APP ZUR EIGENWERTBERECHNUNG AUFRUFEN  *)

   SEIG_APP( RITZMAT , HEIGVAL , HEIGVEC );

   (*  EIGENWERTE SORTIEREN  *)

   K := 0;
   REPEAT VALID := TRUE;
          K := K + 1;
          FOR I := ANZ DOWNTO K + 1 DO
              IF HEIGVAL[I] < HEIGVAL[I-1] THEN BEGIN
                 VALID := FALSE;
                 H1 := HEIGVAL[I];
              (* VR1 := HEIGVEC[*,I]; *)
                 HEIGVAL[I] := HEIGVAL[I-1];
              (* HEIGVEC[*,I] := HEIGVEC[*,I-1]; *)
                 HEIGVAL[I-1] := H1;
              (* HEIGVEC[*,I-1] := VR1; *)
              END;
   UNTIL VALID;

   FOR I := 1 TO ANZ DO BEGIN
       EIGVAL[2*I-1] := HEIGVAL[I];
   (*  FOR J := 0 TO N DO                     *)
   (*      EIGVEC[J,2*I-1] := HEIGVEC[J+1,I]; *)
   END;

   EWT_RITZ := EIGVAL;

END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EW_TRIG_RITZ                                      *)
(*                                                                          *)
(*  RITZ-VERFAHREN ZUR BESTIMMUNG DER SLP-EIGENWERTE EINER                  *)
(*  ENDLICHEN COSINUSREIHE MIT GERADZAHLIGEN FREQUENZEN                     *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)



(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EWS_RITZ                                09.02.93  *)
(*                                                                          *)
(*  BERECHNET FR DEN SPLINE SPP NHERUNGEN FR DIE ERSTEN N EIGENWERTE     *)
(*  MIT DEM RITZ-VERFAHREN ZU DEN ANSATZFUNKTIONEN SIN(kx), K = 1,..ANZ     *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

GLOBAL FUNCTION EWS_RITZ( SPP : RSPLINE; N , ANZ : INTEGER ) : RVECTOR[0..N];

   (*  SPLINES WERDEN IMMER IM INTERVALL  [ 0 ,  ]  VORAUSGESETZT          *)

CONST PI = 3.141592653589793;

VAR I , J , K , M         : INTEGER;
    H , H1 , H2 , H3 , H4 : REAL;
    XI , XI1 , XIK , XI1K : REAL;
    VALID                 : BOOLEAN;
    EIGVAL                : RVECTOR[0..N];
    HEIGVAL , VR1         : RVECTOR[1..ANZ];
    RITZVEC               : RVECTOR[0..2*ANZ];
    RITZMAT , HEIGVEC     : RMATRIX[1..ANZ,1..ANZ];

(*  HEIGVEC IST DUMMY-VARIABLE ZUM AUFRUF DER PROZEDUR SEIG  *)

(*--------------------------------------------------------------------------*)

BEGIN

   (*  ANZAHL DER STTZSTELLEN DES SPLINES P(X) IN [ 0 ,  ]  *)

   M := UB( SPP , 2 );

   (* GLOBALE SCHRITTWEITE SETZEN *)

   H := PI / M;

   (*  ERSTER TEILSCHRITT : INTEGRALE VON  P(X) * COS( 2KX )  BERECHNEN  *)
   (*  ES WIRD AUSSERDEM MIT DEM FAKTOR  NORMIERT, SO DASS ES FR DAS   *)
   (*  RITZ-VERFAHREN PASST                                              *)

   H3 := 0;

   FOR I := 0 TO M - 1 DO BEGIN
       H3 := H3 + H * SPP[0,I] + SQR( H ) * SPP[1,I] / 2;
       H3 := H3 + ( H ** 3 ) * SPP[2,I] / 6 + ( H ** 4 ) * SPP[3,I] / 24;
   END; 
   RITZVEC[0] := H3 / PI;
   
   FOR K := 1 TO 2 * ANZ DO BEGIN

       H2 := 0;
       
       FOR I := 0 TO M - 1 DO BEGIN
           XI := I / M * PI;
           XI1 := ( I + 1 ) / M * PI;
           XIK := 2 * K * XI;
           XI1K := 2 * K * XI1;
           H2 := H2 - SPP[3,I] * ( COS( XI1K ) - COS( XIK ) ) / ( 2 * K );
           H2 := H2 - SPP[2,I+1] * SIN( XI1K ) + SPP[2,I] * SIN( XIK );
       END;

       H2 := H2 / ( 4 * K );
       H2 := H2 - SPP[1,0];
       H2 := H2 / ( 2 * K * K );

       RITZVEC[K] := H2 / PI;

   END;

   (*  ZWEITER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  *)
   (*  MIT GERADEM INDEX :  RITZ-MATRIX BESETZEN         *)

   (*  IM MATRIXARGUMENT GILT  "I = 2 * I - 1, J = 2 * J - 1"  *)
   (*  IM VEKTORARGUMENT GILT ABER  "I = 2 * I , J = 2 * J" !  *)

   FOR I := 1 TO ANZ DO BEGIN
       RITZMAT[I,I] := SQR( 2 * I - 1 ) + RITZVEC[0] - RITZVEC[2*I-1];
       FOR J := 1 TO I - 1 DO BEGIN
           RITZMAT[I,J] := RITZVEC[I-J] - RITZVEC[I+J-1];
           RITZMAT[J,I] := RITZMAT[I,J];
       END;
   END;

   (*  DRITTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  MIT             *)
   (*  GERADEM INDEX :  MODUL SEIG_APP ZUR EIGENWERTBERECHNUNG AUFRUFEN  *)

   SEIG_APP( RITZMAT , HEIGVAL , HEIGVEC );

   (*  KLEINSTE  N DIV 2 + 1  EIGENWERTE SORTIEREN  *)

   K := 0;
   REPEAT VALID := TRUE;
          K := K + 1;
          FOR I := ANZ DOWNTO K + 1 DO
              IF HEIGVAL[I] < HEIGVAL[I-1] THEN BEGIN
                 VALID := FALSE;
                 H4 := HEIGVAL[I];
                 HEIGVAL[I] := HEIGVAL[I-1];
                 HEIGVAL[I-1] := H4;
              END;
   UNTIL  VALID OR ( K = N DIV 2 + 1 );

   FOR I := 0 TO N DIV 2 DO
       EIGVAL[2*I] := HEIGVAL[I+1];

   (*  ZWEITER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)   *)
   (*  MIT UNGERADEM INDEX :  RITZ-MATRIX BESETZEN        *)

   (*  IM MATRIXARGUMENT GILT  "I = 2 * I , J = 2 * J"  *)
   (*  IM VEKTORARGUMENT GILT  "I = 2 * I , J = 2 * J"  *)

   FOR I := 1 TO ANZ DO BEGIN
       RITZMAT[I,I] := SQR( 2 * I ) + RITZVEC[0] - RITZVEC[2*I];
       FOR J := 1 TO I - 1 DO BEGIN
           RITZMAT[I,J] := RITZVEC[I-J] - RITZVEC[I+J];
           RITZMAT[J,I] := RITZMAT[I,J];
       END;
   END;

   (*  DRITTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  MIT               *)
   (*  UNGERADEM INDEX :  MODUL SEIG_APP ZUR EIGENWERTBERECHNUNG AUFRUFEN  *)

   SEIG_APP( RITZMAT , HEIGVAL , HEIGVEC );

   (*  KLEINSTE  ( N + 1 ) DIV 2  EIGENWERTE SORTIEREN  *)

   K := 0;
   REPEAT VALID := TRUE;
          K := K + 1;
          FOR I := ANZ DOWNTO K + 1 DO
              IF HEIGVAL[I] < HEIGVAL[I-1] THEN BEGIN
                 VALID := FALSE;
                 H4 := HEIGVAL[I];
                 HEIGVAL[I] := HEIGVAL[I-1];
                 HEIGVAL[I-1] := H4;
              END;
   UNTIL  VALID OR ( K = ( N + 1 ) DIV 2 );

   FOR I := 1 TO ( N + 1 ) DIV 2 DO
       EIGVAL[2*I-1] := HEIGVAL[I];

   EWS_RITZ := EIGVAL;

END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EWS_RITZ                                          *)
(*                                                                          *)
(*  BERECHNET FR DEN SPLINE SPP NHERUNGEN FR DIE ERSTEN N EIGENWERTE     *)
(*  MIT DEM RITZ-VERFAHREN ZU DEN ANSATZFUNKTIONEN SIN(kx), K = 1,..ANZ     *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)



(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EPS_RITZ                                02.02.93  *)
(*                                                                          *)
(*  BERECHNET FR DEN SPLINE SPP NHERUNGEN FR DIE ERSTEN N EIGENPAARE     *)
(*  MIT DEM RITZ-VERFAHREN ZU DEN ANSATZFUNKTIONEN SIN(kx), K = 1,..ANZ     *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE EPS_RITZ( SPP : RSPLINE; N , ANZ : INTEGER;
                           VAR EIGVAL : RVECTOR; VAR EIGVEC : RMATRIX );

   (*  SPLINES WERDEN IMMER IM INTERVALL  [ 0 ,  ]  VORAUSGESETZT          *)

   (*  AUSGEGEBEN WERDEN DIE KLEINSTEN N+1 EIGENWERTE IN EIGVAL,            *)
   (*  DIE ZUGEHRIGEN EIGENVEKTOREN IN EIGVEC                              *)
   (*  EIN EIGENVEKTOR BESTEHT AUS DEN KOEFFIZIENTEN DER ENSTRECHENDEN      *)
   (*  ENDLICHEN SINUS-REIHE                                                *)
   
CONST PI = 3.141592653589793;

VAR I , J , K , M         : INTEGER;
    H , H1 , H2 , H3 , H4 : REAL;
    XI , XI1 , XIK , XI1K : REAL;
    VALID                 : BOOLEAN;
    HEIGVAL , VR1         : RVECTOR[1..ANZ];
    RITZVEC               : RVECTOR[0..2*ANZ];
    RITZMAT , HEIGVEC     : RMATRIX[1..ANZ,1..ANZ];

(*--------------------------------------------------------------------------*)

BEGIN

   (*  ANZAHL DER STTZSTELLEN DES SPLINES P(X) IN [ 0 ,  ]  *)

   M := UB( SPP , 2 );

   (* GLOBALE SCHRITTWEITE SETZEN *)

   H := PI / M;

   (*  ERSTER TEILSCHRITT : INTEGRALE VON  P(X) * COS( 2KX )  BERECHNEN  *)
   (*  ES WIRD AUSSERDEM MIT DEM FAKTOR  NORMIERT, SO DASS ES FR DAS   *)
   (*  RITZ-VERFAHREN PASST                                              *)

   H3 := 0;

   FOR I := 0 TO M - 1 DO BEGIN
       H3 := H3 + H * SPP[0,I] + SQR( H ) * SPP[1,I] / 2;
       H3 := H3 + ( H ** 3 ) * SPP[2,I] / 6 + ( H ** 4 ) * SPP[3,I] / 24;
   END; 
   RITZVEC[0] := H3 / PI;
   
   FOR K := 1 TO 2 * ANZ DO BEGIN

       H2 := 0;
       
       FOR I := 0 TO M - 1 DO BEGIN
           XI := I / M * PI;
           XI1 := ( I + 1 ) / M * PI;
           XIK := 2 * K * XI;
           XI1K := 2 * K * XI1;
           H2 := H2 - SPP[3,I] * ( COS( XI1K ) - COS( XIK ) ) / ( 2 * K );
           H2 := H2 - SPP[2,I+1] * SIN( XI1K ) + SPP[2,I] * SIN( XIK );
       END;

       H2 := H2 / ( 4 * K );
       H2 := H2 - SPP[1,0];
       H2 := H2 / ( 2 * K * K );

       RITZVEC[K] := H2 / PI;

   END;

   (*  ZWEITER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  *)
   (*  MIT GERADEM INDEX :  RITZ-MATRIX BESETZEN         *)

   (*  IM MATRIXARGUMENT GILT  "I = 2 * I - 1, J = 2 * J - 1"  *)
   (*  IM VEKTORARGUMENT GILT ABER  "I = 2 * I , J = 2 * J" !  *)

   FOR I := 1 TO ANZ DO BEGIN
       RITZMAT[I,I] := SQR( 2 * I - 1 ) + RITZVEC[0] - RITZVEC[2*I-1];
       FOR J := 1 TO I - 1 DO BEGIN
           RITZMAT[I,J] := RITZVEC[I-J] - RITZVEC[I+J-1];
           RITZMAT[J,I] := RITZMAT[I,J];
       END;
   END;

   (*  DRITTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  MIT             *)
   (*  GERADEM INDEX :  MODUL SEIG_APP ZUR EIGENWERTBERECHNUNG AUFRUFEN  *)

   SEIG_APP( RITZMAT , HEIGVAL , HEIGVEC );

   (*  KLEINSTE  N DIV 2 + 1  EIGENWERTE SORTIEREN  *)

   K := 0;
   REPEAT VALID := TRUE;
          K := K + 1;
          FOR I := ANZ DOWNTO K + 1 DO
              IF HEIGVAL[I] < HEIGVAL[I-1] THEN BEGIN
                 VALID := FALSE;
                 H4 := HEIGVAL[I];
                 VR1 := HEIGVEC[*,I];
                 HEIGVAL[I] := HEIGVAL[I-1];
                 HEIGVEC[*,I] := HEIGVEC[*,I-1];
                 HEIGVAL[I-1] := H4;
                 HEIGVEC[*,I-1] := VR1;
              END;
   UNTIL  VALID OR ( K = N DIV 2 + 1 );

   FOR I := 0 TO N DIV 2 DO BEGIN
       EIGVAL[2*I] := HEIGVAL[I+1];
       FOR J := 0 TO N DO
           EIGVEC[J,2*I] := HEIGVEC[J+1,I+1];
   END;

   (*  ZWEITER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)   *)
   (*  MIT UNGERADEM INDEX :  RITZ-MATRIX BESETZEN        *)

   (*  IM MATRIXARGUMENT GILT  "I = 2 * I , J = 2 * J"  *)
   (*  IM VEKTORARGUMENT GILT  "I = 2 * I , J = 2 * J"  *)

   FOR I := 1 TO ANZ DO BEGIN
       RITZMAT[I,I] := SQR( 2 * I ) + RITZVEC[0] - RITZVEC[2*I];
       FOR J := 1 TO I - 1 DO BEGIN
           RITZMAT[I,J] := RITZVEC[I-J] - RITZVEC[I+J];
           RITZMAT[J,I] := RITZMAT[I,J];
       END;
   END;

   (*  DRITTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  MIT               *)
   (*  UNGERADEM INDEX :  MODUL SEIG_APP ZUR EIGENWERTBERECHNUNG AUFRUFEN  *)

   SEIG_APP( RITZMAT , HEIGVAL , HEIGVEC );

   (*  KLEINSTE  ( N + 1 ) DIV 2  EIGENWERTE SORTIEREN  *)

   K := 0;
   REPEAT VALID := TRUE;
          K := K + 1;
          FOR I := ANZ DOWNTO K + 1 DO
              IF HEIGVAL[I] < HEIGVAL[I-1] THEN BEGIN
                 VALID := FALSE;
                 H4 := HEIGVAL[I];
                 VR1 := HEIGVEC[*,I];
                 HEIGVAL[I] := HEIGVAL[I-1];
                 HEIGVEC[*,I] := HEIGVEC[*,I-1];
                 HEIGVAL[I-1] := H4;
                 HEIGVEC[*,I-1] := VR1;
              END;
   UNTIL  VALID OR ( K = ( N + 1 ) DIV 2 );

   FOR I := 1 TO ( N + 1 ) DIV 2 DO BEGIN
       EIGVAL[2*I-1] := HEIGVAL[I];
       FOR J := 0 TO N DO
           EIGVEC[J,2*I-1] := HEIGVEC[J+1,I];
   END;

END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EPS_RITZ                                          *)
(*                                                                          *)
(*  BERECHNET FR DEN SPLINE SPP NHERUNGEN FR DIE ERSTEN N EIGENPAARE     *)
(*  MIT DEM RITZ-VERFAHREN ZU DEN ANSATZFUNKTIONEN SIN(kx), K = 1,..ANZ     *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)



(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR DEPS_RITZ                               02.02.93  *)
(*                                                                          *)
(*  BERECHNET FR DEN SPLINE SPP NHERUNGEN FR DIE ERSTEN N EIGENWERTE     *)
(*  MIT DEM RITZ-VERFAHREN ZU DEN ANSATZFUNKTIONEN SIN(kx), K = 1,..ANZ,    *)
(*  SOWIE DIE FUNKTIONALMATRIX BEIM NEWTON-VERFAHREN FR DAS INVERSE        *)
(*  DIRICHLETPROBLEM AUS DER DISSERTATION                                   *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE DEPS_RITZ( SPP : RSPLINE; M , N , ANZ : INTEGER;
                            VAR EIGVAL : RVECTOR; VAR FUNKT_MAT : RMATRIX );

   (*  BERECHNET FR DEN SPLINE SPP NHERUNGEN FR DIE ERSTEN N EIGENWERTE  *)
   (*  MIT DEM RITZ-VERFAHREN ZU DEN ANSATZFUNKTIONEN SIN(kx), k = 1,..ANZ  *)
   (*  SPLINES WERDEN IMMER IM INTERVALL  [ 0 ,  ] VORAUSGESETZT           *)

   (*  AUSGEGEBEN WERDEN DIE KLEINSTEN N+1 EIGENWERTE IN EIGVAL UND DIE     *)
   (*  FUNKTIONALMATRIX BEIM NEWTON-VERFAHREN ZUM INVERSEN DP IN FUNKT_MAT  *)
   
   (*  ES WIRD VORAUSGESETZT, DASS DIE RITZMATRIZEN DER BASISSPLINES AUS    *)
   (*  DEN FILES  RITZGn.anz  BZW  RITZUn.anz  EINGELESEN WERDEN KNNEN     *)
   (*  BEI DIESEN MATRIZEN IST DIE DIAGONALE NICHT AN DAS DP ANGEPASST !    *)
   
   (*  M   : M + 1 = ANZAHL DER STTZSTELLEN DES SPLINES P(X)               *)
   (*  N   : N + 1 = ANZAHL DER VORGEGEBENEN EIGENWERTE VON P(X)            *)
   (*                N = ( M - 1 ) DIV 2                                    *)

   (*  ANZ = ANZAHL DER ANSATZFUNKTIONEN BEIM RITZ-VERFAHREN                *)
   (*        (JE FR GERADE UND UNGERADE EIGENWERTE)                        *)

CONST PI = 3.141592653589793;

TYPE MATSARRAY = DYNAMIC ARRAY[*,*,*] OF REAL;

VAR I , J , K             : INTEGER;
    H1                    : REAL;
    VALID                 : BOOLEAN;
    RITZEIGVAL , VR1      : RVECTOR[1..ANZ];
    RITZMAT , RITZEIGVEC  : RMATRIX[1..ANZ,1..ANZ];
    BASIS_RITZMATS        : MATSARRAY[0..N,1..ANZ,1..ANZ];
    FILENAMEG , FILENAMEU : STRING;
    FILEG , FILEU         : TEXT;

(*--------------------------------------------------------------------------*)

BEGIN

   (*  FILENAMEN DER RITZ-FILES BESETZEN  *)

   IF M < 10 THEN BEGIN
      FILENAMEG := 'ritzg0' + IMAGE( M , 1 ) + '.' + IMAGE( ANZ , 2 );
      FILENAMEU := 'ritzu0' + IMAGE( M , 1 ) + '.' + IMAGE( ANZ , 2 );
      END
   ELSE BEGIN
      FILENAMEG := 'ritzg' + IMAGE( M , 2 ) + '.' + IMAGE( ANZ , 2 );
      FILENAMEU := 'ritzu' + IMAGE( M , 2 ) + '.' + IMAGE( ANZ , 2 );
   END;

   RESET( FILEG , FILENAMEG );
   RESET( FILEU , FILENAMEU );

   (*  ERSTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  *)
   (*  MIT GERADEM INDEX :  BASIS-RITZ-MATRIX EINLESEN  *)

   FOR K := 0 TO N DO
       FOR J := 1 TO ANZ DO
           FOR I := 1 TO ANZ DO
               READLN( FILEG , BASIS_RITZMATS[K,I,J] );

   (*  ZWEITER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  *)
   (*  MIT GERADEM INDEX :  RITZ-MATRIX BESETZEN         *)

   RITZMAT := SPP[0,0] * BASIS_RITZMATS[0,*,*];
   FOR I := 1 TO N DO
       RITZMAT := RITZMAT + SPP[0,I] * BASIS_RITZMATS[I,*,*];

   (*  DIAGONALTERM  " - U_J'' U_I "  ADDIEREN  *)

   FOR I := 1 TO 30 DO
       RITZMAT[I,I] := RITZMAT[I,I] + SQR( 2 * I - 1 );

   (*  DRITTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  MIT             *)
   (*  GERADEM INDEX :  MODUL SEIG_APP ZUR EIGENWERTBERECHNUNG AUFRUFEN  *)

   SEIG_APP( RITZMAT , RITZEIGVAL , RITZEIGVEC );

   (*  KLEINSTE  N DIV 2 + 1  EIGENWERTE SORTIEREN  *)

   K := 0;
   REPEAT VALID := TRUE;
          K := K + 1;
          FOR I := ANZ DOWNTO K + 1 DO
              IF RITZEIGVAL[I] < RITZEIGVAL[I-1] THEN BEGIN
                 VALID := FALSE;
                 H1 := RITZEIGVAL[I];
                 VR1 := RITZEIGVEC[*,I];
                 RITZEIGVAL[I] := RITZEIGVAL[I-1];
                 RITZEIGVEC[*,I] := RITZEIGVEC[*,I-1];
                 RITZEIGVAL[I-1] := H1;
                 RITZEIGVEC[*,I-1] := VR1;
              END;
   UNTIL  VALID OR ( K = N DIV 2 + 1 );

   FOR I := 0 TO N DIV 2 DO
       EIGVAL[2*I] := RITZEIGVAL[I+1];

   (*  FUNKTIONALMATRIX BERECHNEN  *)

   FOR I := 0 TO N DIV 2 DO
       FOR J := 0 TO N DO
           FUNKT_MAT[2*I,J] := RITZEIGVEC[*,I+1] *
                               ( BASIS_RITZMATS[J,*,*] * RITZEIGVEC[*,I+1] );

   (*  ERSTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)    *)
   (*  MIT UNGERADEM INDEX :  BASIS-RITZ-MATRIX EINLESEN  *)

   FOR K := 0 TO N DO
       FOR J := 1 TO ANZ DO
           FOR I := 1 TO ANZ DO
               READLN( FILEU , BASIS_RITZMATS[K,I,J] );

   (*  ZWEITER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  *)
   (*  MIT UNGERADEM INDEX :  RITZ-MATRIX BESETZEN       *)

   RITZMAT := SPP[0,0] * BASIS_RITZMATS[0,*,*];
   FOR I := 1 TO N DO
       RITZMAT := RITZMAT + SPP[0,I] * BASIS_RITZMATS[I,*,*];

   (*  DIAGONALTERM  " - U_J'' U_I "  ADDIEREN  *)

   FOR I := 1 TO 30 DO
       RITZMAT[I,I] := RITZMAT[I,I] + SQR( 2 * I );

   (*  DRITTER TEILSCHRITT FR DIE EIGENWERTE VON  P(X)  MIT               *)
   (*  UNGERADEM INDEX :  MODUL SEIG_APP ZUR EIGENWERTBERECHNUNG AUFRUFEN  *)

   SEIG_APP( RITZMAT , RITZEIGVAL , RITZEIGVEC );

   (*  KLEINSTE  ( N + 1 ) DIV 2  EIGENWERTE SORTIEREN  *)

   K := 0;
   REPEAT VALID := TRUE;
          K := K + 1;
          FOR I := ANZ DOWNTO K + 1 DO
              IF RITZEIGVAL[I] < RITZEIGVAL[I-1] THEN BEGIN
                 VALID := FALSE;
                 H1 := RITZEIGVAL[I];
                 VR1 := RITZEIGVEC[*,I];
                 RITZEIGVAL[I] := RITZEIGVAL[I-1];
                 RITZEIGVEC[*,I] := RITZEIGVEC[*,I-1];
                 RITZEIGVAL[I-1] := H1;
                 RITZEIGVEC[*,I-1] := VR1;
              END;
   UNTIL  VALID OR ( K = ( N + 1 ) DIV 2 );

   FOR I := 1 TO ( N + 1 ) DIV 2 DO
       EIGVAL[2*I-1] := RITZEIGVAL[I];

   (*  FUNKTIONALMATRIX BERECHNEN  *)

   FOR I := 1 TO ( N + 1 ) DIV 2 DO
       FOR J := 0 TO N DO
           FUNKT_MAT[2*I-1,J] := RITZEIGVEC[*,I] *
                                 ( BASIS_RITZMATS[J,*,*] * RITZEIGVEC[*,I] );

END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR DEPS_RITZ                                         *)
(*                                                                          *)
(*  BERECHNET FR DEN SPLINE SPP NHERUNGEN FR DIE ERSTEN N EIGENWERTE     *)
(*  MIT DEM RITZ-VERFAHREN ZU DEN ANSATZFUNKTIONEN SIN(kx), K = 1,..ANZ,    *)
(*  SOWIE DIE FUNKTIONALMATRIX BEIM NEWTON-VERFAHREN FR DAS INVERSE        *)
(*  DIRICHLETPROBLEM AUS DER DISSERTATION                                   *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)



(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EW_APP                                  02.02.93  *)
(*                                                                          *)
(*  BERECHNET FR DIE FUNKTION TRVECP NHERUNGEN FR EINEN EIGENWERT        *)
(*  IN DER NHE DES EINGABEPARAMETERS EIG                                   *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME MIT AWA GELST                                      *)
(*  ETWAS GENAUER, ABER VOR ALLEM LANGSAMER ALS DAS RITZ-VERFAHREN          *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

GLOBAL FUNCTION EW_APP( M , P , EW , DIVI : INTEGER; TRVECP : RTVEC;
       EIG : REAL; VAR AUS : TEXT ) : REAL;

       (*  P  :  ORDNUNG DER TAYLORENTWICKLUNG VON U  *)
       (*  M  :  SCHRITTWEITE  H = PI / M             *)
       (*  EW :  NUMMER DES EIGENWERTS                *)
       (*  TRVECP ENTHLT DIE TAYLORKOEFFIZIENTEN DES SPLINES  *)
       (*  IN DEN STTZSTELLEN (GEEIGNET NORMIERT)             *)


VAR DIVI_2 , COUNT             : INTEGER;
    EIGALT , STEIGUNG , PI , H : REAL;
    VRU , VRV                  : RVECTOR[1..2];

(*SUB-*)
PROCEDURE DRUCK_AWA; (*-----------------------------------------------------*)
  
(*  DRUCKT ERGEBNISSE DER ANFANGSWERTPROBLEME  *)

VAR J : INTEGER;

BEGIN
   WRITELN( AUS,'Funktionswerte von u(x) am rechten Rand fr  =',EIG,' :');
   WRITELN( AUS );
   WRITELN( AUS , 'U (' , DIVI : 1 , ') = ' , VRU[1] );
   WRITELN( AUS , 'U''(' , DIVI : 1 , ') = ' , VRU[2] );
   WRITELN( AUS );
END;         

(*--- HAUPTPROGRAMM EW_APP -------------------------------------------------*)

BEGIN 

   (*  FR DEN EIGENWERT EW WIRD MIT DEM SEKANTEN-  *)
   (*  VERFAHREN EINE GUTE NHERUNG BERECHNET       *)

   (* GLOBALE SCHRITTWEITE SETZEN *)

   PI:= 3.141592653589793;
   H := PI / M;
   DIVI_2 := SQR( DIVI );

   COUNT := 0;
   WRITE(   'Startnherung des ' , EW : 1 , '. Eigenwerts : ' );
   WRITELN( EIG );
   WRITELN( AUS );
   WRITE(   AUS, '---  Startnherung des ' ,  EW : 1 , '. Eigenwerts : ' );
   WRITELN( AUS , EIG , '  --------------' );
   WRITELN( AUS );
   WRITELN( AUS );
    
   (*  ZUM EINSTIEG IN DAS SEKANTENVEFAHREN                         *)
   (*  FUNKTIONSWERTE IN DER NHE VON EIG BERECHNEN                 *)
   (*  EIGALT ENTHLT DEN WERT DES EIGENWERTS IM "LETZTEN" SCHRITT  *)

(* EIGALT := 1.001 * EIG; *)
   EIGALT := 1.000001 * EIG; 

   (*  ZUM START DES SEKANTENVERFAHRENS WIRD DER FUNKTIONSWERT  *)
   (*  AN DER STELLE EIGALT (IN DER NHE VON EIG BERECHNET      *)

   VRU := AWA( M * DIVI , P , H , EIGALT / DIVI_2 , TRVECP );
    
   (* VRV ENTHLT DEN "FUNKTIONSWERT" DES "LETZTEN" SCHRITTS *)

   VRV := VRU;

   (* JETZT STARTET DAS SEKANTENVERFAHREN *)
         
   REPEAT

      COUNT := COUNT + 1;

      VRU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TRVECP );
      DRUCK_AWA;

      STEIGUNG := ( VRU[1] - VRV[1] ) / ( EIG - EIGALT );
      EIGALT := EIG;
      IF STEIGUNG <> 0 THEN
         EIG := EIG - VRU[1] / STEIGUNG;
      VRV := VRU;
      
      WRITELN( COUNT : 1 , '. Nherung : ' , EIG );
      WRITE(   AUS, COUNT : 1 , '. Nherung des ' );
      WRITELN( AUS , EW : 1 , '. Eigenwerts : ' , EIG );
      WRITELN( AUS );
    
   UNTIL ( ( ABS( EIG - EIGALT ) < 3.0E-12 * ABS( EIGALT ) )
         OR ( STEIGUNG = 0 ) );

   VRU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TRVECP );
   DRUCK_AWA;  

   EW_APP := EIG;

END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EW_APP                                            *)
(*                                                                          *)
(*  BERECHNET FR DIE FUNKTION TRVECP NHERUNGEN FR EINEN EIGENWERT        *)
(*  IN DER NHE DES EINGABEPARAMETERS EIG                                   *)
(*                                                                          *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME MIT AWA GELST                                      *)
(*  ETWAS GENAUER, ABER VOR ALLEM LANGSAMER ALS DAS RITZ-VERFAHREN          *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

                                                                           

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EW_BDS                                  12.02.93  *)
(*                                                                          *)
(*  BERECHNET FR DIE FUNKTION TRVECP SCHRANKEN FR EINEN EIGENWERT         *)
(*  IN DER NHE DES EINGABEPARAMETERS EIG                                   *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME MIT AWA GELST                                      *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

GLOBAL FUNCTION EW_BDS( M, P, EW, DIVI : INTEGER; TIVECP, TIVECP_RST : ITVEC;
                EIG_APP : INTERVAL; PREC : REAL; VAR AUS : TEXT ) : INTERVAL;

       (*  P  :  ORDNUNG DER TAYLORENTWICKLUNG VON U  *)
       (*  M  :  SCHRITTWEITE  H = PI / M             *)
       (*  EW :  NUMMER DES EIGENWERTS                *)
       (*  PREC : GENAUIGKEITSANFORDERUNGEN IN %      *)
       (*  TIVECP ENTHLT DIE TAYLORKOEFFIZIENTEN DES SPLINES  *)
       (*  IN DEN STTZSTELLEN (GEEIGNET NORMIERT)             *)


VAR COUNT                               : INTEGER;
    EIG , STEIGUNG , INCREASE           : REAL;
    EIGLOW , EIGHIGH , FCULOW , FCUHIGH : INTERVAL;
    PI , H , DIVI_2 , EIGBDS            : INTERVAL;
    VIU                                 : IVECTOR[1..2];

(*SUB-*)
PROCEDURE DRUCK( C1 , C2 : CHAR );  (*--------------------------------------*)

(*  HUFIG VERWENDETER STANDARD-AUSDRUCK VON ERGEBNISSEN  *)

BEGIN WRITE( EIG );
   WRITE( AUS , '>' , EIG  );
   IF C1 = 'k' THEN BEGIN
      WRITE( '  ist keine garantierte ' );
      WRITE( AUS , '  ist keine garantierte ' );
      END
   ELSE BEGIN
      WRITE( '  ist eine garantierte ' );
      WRITE( AUS , '  ist eine garantierte ' );
   END;
   IF C2 = 'o' THEN BEGIN
      WRITELN( 'obere Schranke des ' , EW : 1 , '. EW' );
      WRITELN( AUS , 'obere Schranke des ' , EW : 1 , '. EW' );
      END
   ELSE BEGIN
      WRITELN( 'untere Schranke des ' , EW : 1 , '. EW' );
      WRITELN( AUS , 'untere Schranke des ' , EW : 1 , '. EW' );
   END;
   WRITELN( AUS );

END;

(*SUB-*)
PROCEDURE DRUCK_BDS( I : INTEGER );  (*-------------------------------------*)

BEGIN

   WRITELN;
   IF I = 1 THEN BEGIN
      WRITE( 'Grobeinschlieung der Eigenwertschranken nach ' );
      WRITELN( COUNT : 1 , ' Funktionsauswertungen :' );
      END
   ELSE
      WRITELN( COUNT : 1 , '. Einschlieung der Eigenwertschranken :' );
   WRITELN;
   IWRITELN( EIGLOW ); 
   IWRITELN( EIGHIGH );
   WRITELN;

   IF I = 1 THEN BEGIN
      WRITE(   AUS , 'Grobeinschlieung der Eigenwertschranken nach ' );
      WRITELN( AUS , COUNT : 1 , ' Funktionsauswertungen :' );
      END
   ELSE
      WRITELN( AUS, COUNT : 1 , '. Einschlieung der Eigenwertschranken :' );
   WRITELN( AUS );
   IWRITELN( AUS , EIGLOW );
   IWRITELN( AUS , EIGHIGH );
   WRITELN( AUS );

END;

(*SUB-*)
PROCEDURE DRUCK_AWA;  (*----------------------------------------------------*)
  
(*  DRUCKT ERGEBNISSE DER ANFANGSWERTPROBLEME  *)

VAR J : INTEGER;

BEGIN
   WRITELN( AUS,'Funktionswerte von u(x) am rechten Rand fr  =',EIG,' :');
   WRITELN( AUS );
   WRITE( AUS , 'U (' , DIVI : 1 , ') = ' );
   IWRITELN( AUS , VIU[1] );
   WRITE( AUS , 'U''(' , DIVI : 1 , ') = ' );
   IWRITELN( AUS , VIU[2] );
   WRITELN( AUS );
END;         

(*SUB-*)
PROCEDURE IST_USCHRANKE;  (*------------------------------------------------*)
 
(*  DIE AKTUELLE NHERUNG IST EINE GARANTIERTE UNTERSCHRANKE  *) 
(*  DES EIGENWERTS                                            *)
          
BEGIN DRUCK( ' ' , 'u' );

   EIGLOW.INF := EIG;
   FCULOW.INF := VIU[1].INF;
          
END;      
 
(*SUB-*)
PROCEDURE KEINE_USCHRANKE;  (*----------------------------------------------*)

(*  DIE AKTUELLE NHERUNG IST KEINE GARANTIERTE *)
(*  UNTERSCHRANKE DES EIGENWERTS MEHR           *)

BEGIN DRUCK( 'k' , 'u' );

   EIGLOW.SUP := EIG;
   FCULOW.SUP := VIU[1].INF;
   
END;
 
(*SUB-*)
PROCEDURE IST_OSCHRANKE;  (*------------------------------------------------*)

BEGIN DRUCK( ' ' , 'o' );

   EIGHIGH.SUP := EIG;
   FCUHIGH.SUP := VIU[1].SUP;

END;

(*SUB-*)
PROCEDURE KEINE_OSCHRANKE;  (*----------------------------------------------*)

BEGIN DRUCK( 'k' , 'o' );

   EIGHIGH.INF := EIG;
   FCUHIGH.INF := VIU[1].SUP;
                
END;

(*SUB-*)
FUNCTION VALID : BOOLEAN;  (*-----------------------------------------------*)
 
(*  ABBRUCHBEDINGUNG BEI DER ITERATIONSSCHLEIFE  *)
 
BEGIN VALID := TRUE;

   (*  ITERATION BIS ZUR FAST MAXIMAL GENAUEN EINSCHLIESSUNG  *)
   (*  ABBRUCH, FALLS MAXIMAL GENAUE EINSCHLIESSUNG ODER      *)
   (*  FALLS DIE "USSERE" EINSCHLIESSUNG DIE "INNERE" NUR    *)
   (*  GERINGFGIG BERSCHTZT (PARAMETER PREC!)              *)

   IF ( ( PREC > 0 ) AND ( PREC < 1 ) AND
      ( EIGHIGH.INF -> EIGLOW.SUP > PREC *< ( EIGHIGH.SUP -< EIGLOW.INF ) ) )
      THEN VALID := FALSE;

   (*  BEI GANZ ENGER EINSCHLIESSUNG ZUSTZLICHES KRITERIUM NTIG  *)

   IF ( EIGLOW.INF  +> 1.0E-99 >= EIGLOW.SUP  -< 1.0E-99 ) AND
      ( EIGHIGH.INF +> 1.0E-99 >= EIGHIGH.SUP -< 1.0E-99 ) THEN
      VALID := FALSE;
    
END;

(*--- HAUPTPROGRAMM EW_BDS -------------------------------------------------*)

BEGIN

   (*  FR DEN EIGENWERT WERDEN MIT DEM BISEKTIONSVERFAHREN     *)
   (*  ERST EINE UNTER-, DANN EINE OBERSCHRANKE BERECHNET       *)

   (*  DA DIE "FUNKTIONSWERTE" DER "EIGENFUNKTIONEN" WEGEN DEN  *)
   (*  AUFTRETENDEN RUNDUNGSFEHLERN UNSTETIG VON DEN EIGENWERT- *)
   (*  NHERUNGEN ABHNGEN, IST DAS SEKANTENVERFAHREN HIER      *)
   (*  NICHT BRAUCHBAR --> REGULA FALSI                         *)

   (*  ZUR BERECHNUNG EINER GROBEINSCHLIESSUNG IST JEWEILS      *)
   (*  EINE VORITERATION IN DER NHE DES EIGENWERTS NTIG       *)
   (*  ES WIRD VORAUSGESETZT, DASS EINE GUTE NHERUNG DES       *)  
   (*  EIGENWERTS EINGELESEN WIRD                               *)

   (*  GLOBALE SCHRITTWEITE SETZEN  *)

   PI:= INTVAL( RVAL('3.141592653589793',-1) , RVAL('3.141592653589794',1) );
   H := PI / M;
   DIVI_2 := SQR( DIVI );
   EIG := MID( EIG_APP );
   FCULOW := 0;
   FCUHIGH := 0;

   COUNT := 1;
   WRITE(   'Startnherung des ' , EW : 1 , '. Eigenwerts : ' );
   WRITELN( EIG );
   WRITELN( AUS );
   WRITE(   AUS, '---  Startnherung des ' ,  EW : 1 , '. Eigenwerts : ' );
   WRITELN( AUS , EIG , '  --------------' );
   WRITELN( AUS );
   WRITELN( AUS );
    
   (*  SCHRITTWEITE BEI DER VORITERATION MINDESTENS     *)
   (*  HALBER DURCHMESSER DER EIGENWERTANFANGSNHERUNG  *)
   
   INCREASE := ( 1 + ABS( EIG ) ) * 2.0E-14;
   INCREASE := INCREASE + ( EIG_APP.SUP - EIG_APP.INF ) / 2;

   WRITE(   AUS, 'Mindestschrittweite bei der Voriteration : ' );
   WRITELN( AUS , INCREASE );
   WRITELN( AUS );

   (*  ZUM EINSTIEG IN DAS BISEKTIONSVEFAHREN  *)
   (*  FUNKTIONSWERTE FR EIG BERECHNEN        *)

   VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
   VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
   DRUCK_AWA;

   (*  ZUERST GROBEINSCHLIESSUNG DER UNTERSCHRANKE BERECHNEN *)
     
   IF VIU[1].INF >= 0 THEN BEGIN
    
      IST_USCHRANKE;
      KEINE_OSCHRANKE;
      
      (*  JETZT IST EINE NICHT MEHR GARANTIERBARE (ZU GROSSE)  *)
      (*  UNTERSCHRANKE ZU BERECHNEN                           *)
       
      REPEAT
        
         (*  HEURISTISCHE ERHHUNG UM DEN DURCHMESSER DES SPLINES  *)
         (*  UND DEN WERT ABS( U() ) / ABS( VIU[2].SUP )          *)
         
         COUNT := COUNT + 1;

         EIG := EIG + INCREASE + VIU[1].INF / ABS( VIU[2].SUP );
         INCREASE := 2 * INCREASE;

         VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].INF >= 0 THEN BEGIN
            IST_USCHRANKE;
            KEINE_OSCHRANKE;
            END
         ELSE BEGIN
            KEINE_USCHRANKE;
            IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE
            ELSE KEINE_OSCHRANKE;
            END;

      UNTIL VIU[1].INF < 0;
       
      END

   ELSE BEGIN

      KEINE_USCHRANKE;
      IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE
      ELSE KEINE_OSCHRANKE;
       
      REPEAT
        
         (*  HEURISTISCHE VERMINDERUNG UM DEN DURCHMESSER DES SPLINES  *)
         (*  UND DEN WERT ABS( U() ) / ABS( VIU[2].SUP )              *)
         
         COUNT := COUNT + 1;
                                 
         EIG := EIG - INCREASE + VIU[1].INF / ABS( VIU[2].SUP );
         INCREASE := 2 * INCREASE;

         VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].INF >= 0 THEN BEGIN
            IST_USCHRANKE;
            IF FCUHIGH.INF = 0 THEN KEINE_OSCHRANKE;(* -> NOCH KEINE UNTER-*)
            END                                       (* SCHRANKE DER OBER-*)
         ELSE BEGIN                                   (* SCHRANKE GEFUNDEN *)
            KEINE_USCHRANKE;
            IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE
            ELSE IF FCUHIGH.INF = 0 THEN KEINE_OSCHRANKE;  (* -^  *)
            END;

      UNTIL VIU[1].INF >= 0;

   END;

   (*  EVENTUELL MUSS NOCH EINE GARANTIERTE  *)
   (*  OBERSCHRANKE BERECHNET WERDEN         *)

   IF FCUHIGH.SUP = 0 THEN BEGIN

      (*  STARTWERTE VORBESETZEN *)

      INCREASE := ( 1 + ABS( EIG ) ) * 2.0E-14;
      INCREASE := INCREASE + ( EIG_APP.SUP - EIG_APP.INF ) / 2;

      EIG := EIGHIGH.INF;
      VIU[1].SUP := FCUHIGH.INF;
      VIU[2].SUP := 1.0E+14 * VIU[1].SUP;  (*  DUMMY-WERT  *)

      (*  JETZT IST EINE GARANTIERTE OBERSCHRANKE ZU BERECHNEN  *)

      REPEAT
        
         (*  HEURISTISCHE ERHHUNG UM DEN DURCHMESSER DES SPLINES  *)
         (*  UND DEN WERT ABS( U() ) / ABS( VIU[2].SUP )          *)
         
         COUNT := COUNT + 1;

         EIG := EIG + INCREASE + VIU[1].SUP / ABS( VIU[2].SUP );
         INCREASE := 2 * INCREASE;

         VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE

         ELSE KEINE_OSCHRANKE;

      UNTIL VIU[1].SUP <= 0;

   END;

   DRUCK_BDS( 1 );

   (*  SOLANGE DIE EINSCHLIESSUNGEN DER UNTER- UND DER OBERSCHRANKE        *)
   (*  BEREINSTIMMEN, KANN SIMULTAN MIT DER REGULA FALSI ITERIERT WERDEN  *)

   WHILE EIGHIGH = EIGLOW DO BEGIN

         COUNT := COUNT + 1;

         STEIGUNG:= ( FCUHIGH.SUP-FCULOW.INF ) / ( EIGHIGH.SUP-EIGLOW.INF );
         (*  STEIGUNG IST <> 0 , WEIL FCUHIGH.SUP * FCULOW.INF <> 0  *)

         EIG := EIGLOW.INF - FCULOW.INF / STEIGUNG;

         (*  HEURISTISCHE KORREKTUR BEI ALLZU UNSYMMETRISCHER TEILUNG  *)

         IF ( EIG - EIGLOW.INF ) < ( EIGHIGH.SUP - EIGLOW.INF ) / 20 THEN
            EIG := EIGLOW.INF -> 2 *> FCULOW.INF /> STEIGUNG;

         IF ( EIGHIGH.SUP - EIG ) < ( EIGHIGH.SUP - EIGLOW.INF ) / 20 THEN
            EIG := 2 * EIG -< EIGHIGH.SUP;

         VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].INF >= 0 THEN BEGIN
            IST_USCHRANKE;
            KEINE_OSCHRANKE;
            END
         ELSE IF VIU[1].SUP <= 0 THEN BEGIN
            IST_OSCHRANKE;
            KEINE_USCHRANKE;
            END
         ELSE BEGIN
            KEINE_USCHRANKE;
            KEINE_OSCHRANKE;
            END;

         DRUCK_BDS( 0 );

   END;  (* WHILE *)

   DRUCK_BDS( 1 );
   
   (*  DIE GROBEINSCHLIESSUNGEN EIGLOW UND EIGHIGH SIND JETZT  *)
   (*  MIT DER REGULA FALSI SEPARAT ZU VERBESSERN              *)

   COUNT := 1;
    
   WHILE VALID DO BEGIN
      
      IF EIGLOW.SUP - EIGLOW.INF >= EIGHIGH.SUP - EIGHIGH.INF THEN BEGIN

         (*  UNTERSCHRANKE VERBESSERN                             *)
         (*  IN DER BEDINGUNG STEHT  ">=" , DAMIT BEI GLEICHHEIT  *)
         (*  KEINE ENDLOSSCHLEIFE DURCHLAUFEN WIRD                *)

         COUNT := COUNT + 1;

         STEIGUNG:= ( FCULOW.SUP-FCULOW.INF ) / ( EIGLOW.SUP-EIGLOW.INF );
         (*  STEIGUNG IST <> 0 , WEIL FCULOW.SUP * FCULOW.INF <> 0  *)

         EIG := EIGLOW.INF - FCULOW.INF / STEIGUNG;

         (*  HEURISTISCHE KORREKTUR BEI ALLZU UNSYMMETRISCHER TEILUNG  *)

         IF ( EIG - EIGLOW.INF ) < ( EIGLOW.SUP - EIGLOW.INF ) / 20 THEN
            EIG := EIGLOW.INF - 2 * FCULOW.INF / STEIGUNG;

         IF ( EIGLOW.SUP - EIG ) < ( EIGLOW.SUP - EIGLOW.INF ) / 20 THEN
            EIG := 2 * EIG - EIGLOW.SUP;

         IF EIG <= EIGLOW.INF THEN
            EIG := EIGLOW.INF +> ( EIGLOW.SUP - EIGLOW.INF ) /> 4;

         IF EIG >= EIGLOW.SUP THEN
            EIG := EIGLOW.SUP -< ( EIGLOW.SUP - EIGLOW.INF ) /> 4;

         VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].INF >= 0 THEN IST_USCHRANKE

         ELSE KEINE_USCHRANKE;

         DRUCK_BDS( 0 );

      END; (* IF *)
       
      IF ( VALID AND (EIGHIGH.SUP-EIGHIGH.INF >= EIGLOW.SUP-EIGLOW.INF) )
         THEN BEGIN

         (*  OBERSCHRANKE VERBESSERN  *)

         COUNT := COUNT + 1;

         STEIGUNG:= (FCUHIGH.SUP-FCUHIGH.INF) / (EIGHIGH.SUP-EIGHIGH.INF);
         (*  STEIGUNG IST <> 0 , WEIL FCUHIGH.SUP * FCUHIGH.INF <> 0  *)

         EIG := EIGHIGH.INF - FCUHIGH.INF / STEIGUNG;

         (*  HEURISTISCHE KORREKTUR BEI ALLZU UNSYMMETRISCHER TEILUNG  *)

         IF ( EIG - EIGHIGH.INF ) < ( EIGHIGH.SUP - EIGHIGH.INF )/ 20 THEN
            EIG := EIGHIGH.INF - 2 * FCUHIGH.INF / STEIGUNG;

         IF ( EIGHIGH.SUP - EIG ) < ( EIGHIGH.SUP - EIGHIGH.INF )/ 20 THEN
            EIG := 2 * EIG - EIGHIGH.SUP;

         IF EIG <= EIGHIGH.INF THEN
            EIG := EIGHIGH.INF +> ( EIGHIGH.SUP - EIGHIGH.INF ) /> 4;

         IF EIG >= EIGHIGH.SUP THEN
            EIG := EIGHIGH.SUP -< ( EIGHIGH.SUP - EIGHIGH.INF ) /> 4;

         VIU := AWA( M * DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE

         ELSE KEINE_OSCHRANKE;

         DRUCK_BDS( 0 );

      END; (* IF *)

   END; (* WHILE *)
       
   EIGBDS.INF := EIGLOW.INF;
   EIGBDS.SUP := EIGHIGH.SUP;

   EW_BDS := EIGBDS;

END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EW_BDS                                            *)
(*                                                                          *)
(*  BERECHNET FR DIE FUNKTION TRVECP SCHRANKEN FR EINEN EIGENWERT         *)
(*  IN DER NHE DES EINGABEPARAMETERS EIG                                   *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME MIT AWA GELST                                      *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

                                                                           

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EWS_BDS                                 02.03.93  *)
(*                                                                          *)
(*  BERECHNET FR DEN SPLINE TRVECP SCHRANKEN FR EINEN EIGENWERT           *)
(*  IN DER NHE DES EINGABEPARAMETERS EIG                                   *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME MIT AWA_S GELST                                    *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

GLOBAL FUNCTION EWS_BDS( M, P, EW, DIVI : INTEGER; TIVECP, TIVECP_RST : ITVEC;
                EIG_APP : INTERVAL; PREC : REAL; VAR AUS : TEXT ) : INTERVAL;

       (*  P  :  ORDNUNG DER TAYLORENTWICKLUNG VON U  *)
       (*  M  :  SCHRITTWEITE  H = PI / M             *)
       (*  EW :  NUMMER DES EIGENWERTS                *)
       (*  PREC : GENAUIGKEITSANFORDERUNGEN IN %      *)
       (*  TIVECP ENTHLT DIE TAYLORKOEFFIZIENTEN DES SPLINES  *)
       (*  IN DEN STTZSTELLEN (GEEIGNET NORMIERT)             *)


VAR COUNT                               : INTEGER;
    EIG , STEIGUNG , INCREASE           : REAL;
    EIGLOW , EIGHIGH , FCULOW , FCUHIGH : INTERVAL;
    PI , H , DIVI_2 , EIGBDS            : INTERVAL;
    VIU                                 : IVECTOR[1..2];

(*SUB-*)
PROCEDURE DRUCK( C1 , C2 : CHAR );  (*--------------------------------------*)

(*  HUFIG VERWENDETER STANDARD-AUSDRUCK VON ERGEBNISSEN  *)

BEGIN WRITE( EIG );
   WRITE( AUS , '>' , EIG  );
   IF C1 = 'k' THEN BEGIN
      WRITE( '  ist keine garantierte ' );
      WRITE( AUS , '  ist keine garantierte ' );
      END
   ELSE BEGIN
      WRITE( '  ist eine garantierte ' );
      WRITE( AUS , '  ist eine garantierte ' );
   END;
   IF C2 = 'o' THEN BEGIN
      WRITELN( 'obere Schranke des ' , EW : 1 , '. EW' );
      WRITELN( AUS , 'obere Schranke des ' , EW : 1 , '. EW' );
      END
   ELSE BEGIN
      WRITELN( 'untere Schranke des ' , EW : 1 , '. EW' );
      WRITELN( AUS , 'untere Schranke des ' , EW : 1 , '. EW' );
   END;
   WRITELN( AUS );

END;

(*SUB-*)
PROCEDURE DRUCK_BDS( I : INTEGER );  (*-------------------------------------*)

BEGIN

   WRITELN;
   IF I = 1 THEN BEGIN
      WRITE( 'Grobeinschlieung der Eigenwertschranken nach ' );
      WRITELN( COUNT : 1 , ' Funktionsauswertungen :' );
      END
   ELSE
      WRITELN( COUNT : 1 , '. Einschlieung der Eigenwertschranken :' );
   WRITELN;
   IWRITELN( EIGLOW ); 
   IWRITELN( EIGHIGH );
   WRITELN;

   IF I = 1 THEN BEGIN
      WRITE(   AUS , 'Grobeinschlieung der Eigenwertschranken nach ' );
      WRITELN( AUS , COUNT : 1 , ' Funktionsauswertungen :' );
      END
   ELSE
      WRITELN( AUS, COUNT : 1 , '. Einschlieung der Eigenwertschranken :' );
   WRITELN( AUS );
   IWRITELN( AUS , EIGLOW );
   IWRITELN( AUS , EIGHIGH );
   WRITELN( AUS );

END;

(*SUB-*)
PROCEDURE DRUCK_AWA;  (*----------------------------------------------------*)
  
(*  DRUCKT ERGEBNISSE DER ANFANGSWERTPROBLEME  *)

VAR J : INTEGER;

BEGIN
   WRITELN( AUS,'Funktionswerte von u(x) am rechten Rand fr  =',EIG,' :');
   WRITELN( AUS );
   WRITE( AUS , 'U (' , DIVI : 1 , ') = ' );
   IWRITELN( AUS , VIU[1] );
   WRITE( AUS , 'U''(' , DIVI : 1 , ') = ' );
   IWRITELN( AUS , VIU[2] );
   WRITELN( AUS );
END;         

(*SUB-*)
PROCEDURE IST_USCHRANKE;  (*------------------------------------------------*)
 
(*  DIE AKTUELLE NHERUNG IST EINE GARANTIERTE UNTERSCHRANKE  *) 
(*  DES EIGENWERTS                                            *)
          
BEGIN DRUCK( ' ' , 'u' );

   EIGLOW.INF := EIG;
   FCULOW.INF := VIU[1].INF;
          
END;      
 
(*SUB-*)
PROCEDURE KEINE_USCHRANKE;  (*----------------------------------------------*)

(*  DIE AKTUELLE NHERUNG IST KEINE GARANTIERTE *)
(*  UNTERSCHRANKE DES EIGENWERTS MEHR           *)

BEGIN DRUCK( 'k' , 'u' );

   EIGLOW.SUP := EIG;
   FCULOW.SUP := VIU[1].INF;
   
END;
 
(*SUB-*)
PROCEDURE IST_OSCHRANKE;  (*------------------------------------------------*)

BEGIN DRUCK( ' ' , 'o' );

   EIGHIGH.SUP := EIG;
   FCUHIGH.SUP := VIU[1].SUP;

END;

(*SUB-*)
PROCEDURE KEINE_OSCHRANKE;  (*----------------------------------------------*)

BEGIN DRUCK( 'k' , 'o' );

   EIGHIGH.INF := EIG;
   FCUHIGH.INF := VIU[1].SUP;
                
END;

(*SUB-*)
FUNCTION VALID : BOOLEAN;  (*-----------------------------------------------*)
 
(*  ABBRUCHBEDINGUNG BEI DER ITERATIONSSCHLEIFE  *)
 
BEGIN VALID := TRUE;

   (*  ITERATION BIS ZUR FAST MAXIMAL GENAUEN EINSCHLIESSUNG  *)
   (*  ABBRUCH, FALLS MAXIMAL GENAUE EINSCHLIESSUNG ODER      *)
   (*  FALLS DIE "USSERE" EINSCHLIESSUNG DIE "INNERE" NUR    *)
   (*  GERINGFGIG BERSCHTZT (PARAMETER PREC!)              *)

   IF ( ( PREC > 0 ) AND ( PREC < 1 ) AND
      ( EIGHIGH.INF -> EIGLOW.SUP > PREC *< ( EIGHIGH.SUP -< EIGLOW.INF ) ) )
      THEN VALID := FALSE;

   (*  BEI GANZ ENGER EINSCHLIESSUNG ZUSTZLICHES KRITERIUM NTIG  *)

   IF ( EIGLOW.INF  +> 1.0E-99 >= EIGLOW.SUP  -< 1.0E-99 ) AND
      ( EIGHIGH.INF +> 1.0E-99 >= EIGHIGH.SUP -< 1.0E-99 ) THEN
      VALID := FALSE;
    
END;

(*--- HAUPTPROGRAMM EWS_BDS ------------------------------------------------*)

BEGIN

   (*  FR DEN EIGENWERT WERDEN MIT DEM BISEKTIONSVERFAHREN     *)
   (*  ERST EINE UNTER-, DANN EINE OBERSCHRANKE BERECHNET       *)

   (*  DA DIE "FUNKTIONSWERTE" DER "EIGENFUNKTIONEN" WEGEN DEN  *)
   (*  AUFTRETENDEN RUNDUNGSFEHLERN UNSTETIG VON DEN EIGENWERT- *)
   (*  NHERUNGEN ABHNGEN, IST DAS SEKANTENVERFAHREN HIER      *)
   (*  NICHT BRAUCHBAR --> REGULA FALSI                         *)

   (*  ZUR BERECHNUNG EINER GROBEINSCHLIESSUNG IST JEWEILS      *)
   (*  EINE VORITERATION IN DER NHE DES EIGENWERTS NTIG       *)
   (*  ES WIRD VORAUSGESETZT, DASS EINE GUTE NHERUNG DES       *)  
   (*  EIGENWERTS EINGELESEN WIRD                               *)

   (*  GLOBALE SCHRITTWEITE SETZEN  *)

   PI:= INTVAL( RVAL('3.141592653589793',-1) , RVAL('3.141592653589794',1) );
   H := PI / M;
   DIVI_2 := SQR( DIVI );
   EIG := MID( EIG_APP );
   FCULOW := 0;
   FCUHIGH := 0;

   COUNT := 1;
   WRITE(   'Startnherung des ' , EW : 1 , '. Eigenwerts : ' );
   WRITELN( EIG );
   WRITELN( AUS );
   WRITE(   AUS, '---  Startnherung des ' ,  EW : 1 , '. Eigenwerts : ' );
   WRITELN( AUS , EIG , '  --------------' );
   WRITELN( AUS );
   WRITELN( AUS );
    
   (*  SCHRITTWEITE BEI DER VORITERATION MINDESTENS     *)
   (*  HALBER DURCHMESSER DER EIGENWERTANFANGSNHERUNG  *)
   
   INCREASE := ( 1 + ABS( EIG ) ) * 2.0E-14;
   INCREASE := INCREASE + ( EIG_APP.SUP - EIG_APP.INF ) / 2;

   WRITE(   AUS, 'Mindestschrittweite bei der Voriteration : ' );
   WRITELN( AUS , INCREASE );
   WRITELN( AUS );

   (*  ZUM EINSTIEG IN DAS BISEKTIONSVEFAHREN  *)
   (*  FUNKTIONSWERTE FR EIG BERECHNEN        *)

   VIU := AWA_S( M , DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
   VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
   DRUCK_AWA;

   (*  ZUERST GROBEINSCHLIESSUNG DER UNTERSCHRANKE BERECHNEN *)
     
   IF VIU[1].INF >= 0 THEN BEGIN
    
      IST_USCHRANKE;
      KEINE_OSCHRANKE;
      
      (*  JETZT IST EINE NICHT MEHR GARANTIERBARE (ZU GROSSE)  *)
      (*  UNTERSCHRANKE ZU BERECHNEN                           *)
       
      REPEAT
        
         (*  HEURISTISCHE ERHHUNG UM DEN DURCHMESSER DES SPLINES  *)
         (*  UND DEN WERT ABS( U() ) / ABS( VIU[2].SUP )          *)
         
         COUNT := COUNT + 1;

         EIG := EIG + INCREASE + VIU[1].INF / ABS( VIU[2].SUP );
         INCREASE := 2 * INCREASE;

         VIU := AWA_S( M , DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].INF >= 0 THEN BEGIN
            IST_USCHRANKE;
            KEINE_OSCHRANKE;
            END
         ELSE BEGIN
            KEINE_USCHRANKE;
            IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE
            ELSE KEINE_OSCHRANKE;
            END;

      UNTIL VIU[1].INF < 0;
       
      END

   ELSE BEGIN

      KEINE_USCHRANKE;
      IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE
      ELSE KEINE_OSCHRANKE;
       
      REPEAT
        
         (*  HEURISTISCHE VERMINDERUNG UM DEN DURCHMESSER DES SPLINES  *)
         (*  UND DEN WERT ABS( U() ) / ABS( VIU[2].SUP )              *)
         
         COUNT := COUNT + 1;
                                 
         EIG := EIG - INCREASE + VIU[1].INF / ABS( VIU[2].SUP );
         INCREASE := 2 * INCREASE;

         VIU := AWA_S( M , DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].INF >= 0 THEN BEGIN
            IST_USCHRANKE;
            IF FCUHIGH.INF = 0 THEN KEINE_OSCHRANKE;(* -> NOCH KEINE UNTER-*)
            END                                       (* SCHRANKE DER OBER-*)
         ELSE BEGIN                                   (* SCHRANKE GEFUNDEN *)
            KEINE_USCHRANKE;
            IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE
            ELSE IF FCUHIGH.INF = 0 THEN KEINE_OSCHRANKE;  (* -^  *)
            END;

      UNTIL VIU[1].INF >= 0;

   END;

   (*  EVENTUELL MUSS NOCH EINE GARANTIERTE  *)
   (*  OBERSCHRANKE BERECHNET WERDEN         *)

   IF FCUHIGH.SUP = 0 THEN BEGIN

      (*  STARTWERTE VORBESETZEN *)

      INCREASE := ( 1 + ABS( EIG ) ) * 2.0E-14;
      INCREASE := INCREASE + ( EIG_APP.SUP - EIG_APP.INF ) / 2;

      EIG := EIGHIGH.INF;
      VIU[1].SUP := FCUHIGH.INF;
      VIU[2].SUP := 1.0E+14 * VIU[1].SUP;  (*  DUMMY-WERT  *)

      (*  JETZT IST EINE GARANTIERTE OBERSCHRANKE ZU BERECHNEN  *)

      REPEAT
        
         (*  HEURISTISCHE ERHHUNG UM DEN DURCHMESSER DES SPLINES  *)
         (*  UND DEN WERT ABS( U() ) / ABS( VIU[2].SUP )          *)
         
         COUNT := COUNT + 1;

         EIG := EIG + INCREASE + VIU[1].SUP / ABS( VIU[2].SUP );
         INCREASE := 2 * INCREASE;

         VIU := AWA_S( M , DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE

         ELSE KEINE_OSCHRANKE;

      UNTIL VIU[1].SUP <= 0;

   END;

   DRUCK_BDS( 1 );

   (*  SOLANGE DIE EINSCHLIESSUNGEN DER UNTER- UND DER OBERSCHRANKE        *)
   (*  BEREINSTIMMEN, KANN SIMULTAN MIT DER REGULA FALSI ITERIERT WERDEN  *)

   WHILE EIGHIGH = EIGLOW DO BEGIN

         COUNT := COUNT + 1;

         STEIGUNG:= ( FCUHIGH.SUP-FCULOW.INF ) / ( EIGHIGH.SUP-EIGLOW.INF );
         (*  STEIGUNG IST <> 0 , WEIL FCUHIGH.SUP * FCULOW.INF <> 0  *)

         EIG := EIGLOW.INF - FCULOW.INF / STEIGUNG;

         (*  HEURISTISCHE KORREKTUR BEI ALLZU UNSYMMETRISCHER TEILUNG  *)

         IF ( EIG - EIGLOW.INF ) < ( EIGHIGH.SUP - EIGLOW.INF ) / 20 THEN
            EIG := EIGLOW.INF -> 2 *> FCULOW.INF /> STEIGUNG;

         IF ( EIGHIGH.SUP - EIG ) < ( EIGHIGH.SUP - EIGLOW.INF ) / 20 THEN
            EIG := 2 * EIG -< EIGHIGH.SUP;

         VIU := AWA_S( M , DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].INF >= 0 THEN BEGIN
            IST_USCHRANKE;
            KEINE_OSCHRANKE;
            END
         ELSE IF VIU[1].SUP <= 0 THEN BEGIN
            IST_OSCHRANKE;
            KEINE_USCHRANKE;
            END
         ELSE BEGIN
            KEINE_USCHRANKE;
            KEINE_OSCHRANKE;
            END;

         DRUCK_BDS( 0 );

   END;  (* WHILE *)

   DRUCK_BDS( 1 );
   
   (*  DIE GROBEINSCHLIESSUNGEN EIGLOW UND EIGHIGH SIND JETZT  *)
   (*  MIT DER REGULA FALSI SEPARAT ZU VERBESSERN              *)

   COUNT := 1;
    
   WHILE VALID DO BEGIN
      
      IF EIGLOW.SUP - EIGLOW.INF >= EIGHIGH.SUP - EIGHIGH.INF THEN BEGIN

         (*  UNTERSCHRANKE VERBESSERN                             *)
         (*  IN DER BEDINGUNG STEHT  ">=" , DAMIT BEI GLEICHHEIT  *)
         (*  KEINE ENDLOSSCHLEIFE DURCHLAUFEN WIRD                *)

         COUNT := COUNT + 1;

         STEIGUNG:= ( FCULOW.SUP-FCULOW.INF ) / ( EIGLOW.SUP-EIGLOW.INF );
         (*  STEIGUNG IST <> 0 , WEIL FCULOW.SUP * FCULOW.INF <> 0  *)

         EIG := EIGLOW.INF - FCULOW.INF / STEIGUNG;

         (*  HEURISTISCHE KORREKTUR BEI ALLZU UNSYMMETRISCHER TEILUNG  *)

         IF ( EIG - EIGLOW.INF ) < ( EIGLOW.SUP - EIGLOW.INF ) / 20 THEN
            EIG := EIGLOW.INF - 2 * FCULOW.INF / STEIGUNG;

         IF ( EIGLOW.SUP - EIG ) < ( EIGLOW.SUP - EIGLOW.INF ) / 20 THEN
            EIG := 2 * EIG - EIGLOW.SUP;

         IF EIG <= EIGLOW.INF THEN
            EIG := EIGLOW.INF +> ( EIGLOW.SUP - EIGLOW.INF ) /> 4;

         IF EIG >= EIGLOW.SUP THEN
            EIG := EIGLOW.SUP -< ( EIGLOW.SUP - EIGLOW.INF ) /> 4;

         VIU := AWA_S( M , DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].INF >= 0 THEN IST_USCHRANKE

         ELSE KEINE_USCHRANKE;

         DRUCK_BDS( 0 );

      END; (* IF *)
       
      IF ( VALID AND (EIGHIGH.SUP-EIGHIGH.INF >= EIGLOW.SUP-EIGLOW.INF) )
         THEN BEGIN

         (*  OBERSCHRANKE VERBESSERN  *)

         COUNT := COUNT + 1;

         STEIGUNG:= (FCUHIGH.SUP-FCUHIGH.INF) / (EIGHIGH.SUP-EIGHIGH.INF);
         (*  STEIGUNG IST <> 0 , WEIL FCUHIGH.SUP * FCUHIGH.INF <> 0  *)

         EIG := EIGHIGH.INF - FCUHIGH.INF / STEIGUNG;

         (*  HEURISTISCHE KORREKTUR BEI ALLZU UNSYMMETRISCHER TEILUNG  *)

         IF ( EIG - EIGHIGH.INF ) < ( EIGHIGH.SUP - EIGHIGH.INF )/ 20 THEN
            EIG := EIGHIGH.INF - 2 * FCUHIGH.INF / STEIGUNG;

         IF ( EIGHIGH.SUP - EIG ) < ( EIGHIGH.SUP - EIGHIGH.INF )/ 20 THEN
            EIG := 2 * EIG - EIGHIGH.SUP;

         IF EIG <= EIGHIGH.INF THEN
            EIG := EIGHIGH.INF +> ( EIGHIGH.SUP - EIGHIGH.INF ) /> 4;

         IF EIG >= EIGHIGH.SUP THEN
            EIG := EIGHIGH.SUP -< ( EIGHIGH.SUP - EIGHIGH.INF ) /> 4;

         VIU := AWA_S( M , DIVI , P , H , EIG / DIVI_2 , TIVECP , TIVECP_RST );
         VIU := VIU * ( 1 - 2 * ( EW MOD 2 ) );
         DRUCK_AWA;

         IF VIU[1].SUP <= 0 THEN IST_OSCHRANKE

         ELSE KEINE_OSCHRANKE;

         DRUCK_BDS( 0 );

      END; (* IF *)

   END; (* WHILE *)
       
   EIGBDS.INF := EIGLOW.INF;
   EIGBDS.SUP := EIGHIGH.SUP;

   EWS_BDS := EIGBDS;

END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROZEDUR EWS_BDS                                           *)
(*                                                                          *)
(*  BERECHNET FR DEN SPLINE TRVECP SCHRANKEN FR EINEN EIGENWERT           *)
(*  IN DER NHE DES EINGABEPARAMETERS EIG                                   *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME MIT AWA_S GELST                                    *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

END.

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - MODUL  DEP                                                 *)
(*                                                                          *)
(*  BERECHNUNG VON NHERUNGEN UND EINSCHLIESSUNGEN VON EIGENWERTEN UND      *)
(*  EIGENFUNKTIONEN DER STURM-LIOUVILLE-GLEICHUNG ( DIREKTES PROBLEM )      *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)
